function [best_nfa best_curve_pos best_curve_ors all_nfas] = algo_curves(points, orientations)
% [best_nfa best_line_pos best_line_ors all_nfas] = algo_curves(points, orientations)
%
% - points : a N x 2 matrix; each line contains the coordinates of a point
% - orientations : a vector of length N, containing the orientations of the
%   patches
%
% - best_nfa : the smallest found NFA
% - best_curve_pos : the coordinates of the elements of the detected
%   curve
% - all_nfas : the NFA values of all tested chains

N = length(points);% total number of elements
nn = 6; % number of nearest neighbours in  graph
max_chain_length = floor(sqrt(N));% the maximum number of elements of a tested chain
Ntests = nn*N*(max_chain_length-2); % Number of tests

D = dist2(points, points); % matrix of square distances between points
gamma_nn = knn_oriented(D, nn); % adjacence matrix defining the edges of the oriented graph gamma_6


dist_all_edges = zeros(sum(sum(gamma_nn)), 1); % List of the distances of all edges
l = 0;
for i = 1:N,
    for k = 1:N,
        if gamma_nn(i, k),
            l = l+1;
            dist_all_edges(l) = D(i,k);
        end
    end
end

best_nfa = inf;
best_chain = [];
all_nfas = [];
for i = 1 : N,
    % Testing chains starting at point i
    neighbours_of_i = find(gamma_nn(i,:)); % indexes of the neighbours of point i
    for v = 1: length(neighbours_of_i),
        j = neighbours_of_i(v);
        
        chain = [i, j]; % initiate a chain
        last_point = chain(2);
        alphas = [abs( compute_angular_errors(points(chain(1), :), points(chain(2), :), orientations(chain(1))) ), 0];
        length_edges = [D(i,j)]; % variables \ell_i
        while length(chain) < max_chain_length && ~isempty(last_point),
            
            last_point = find_next_point(points, gamma_nn, chain);
            chain = [chain, last_point];
            
            if ~isempty(last_point),
                alphas(end) = abs( compute_angular_errors(points(chain(end-2), :), points(chain(end), :), orientations(chain(end-1))) );
                alphas = [alphas, abs( compute_angular_errors(points(chain(end-1), :), points(chain(end), :), orientations(chain(end))) )];
                length_edges = [length_edges, D(chain(end), chain(end-1))];


%% Sum all normalized variables
                norm_sum_alphas = 2*sum(alphas)/pi;
                F_l = zeros(size(length_edges));
                for d = 1:length(F_l),
                    F_l(d) = sum(dist_all_edges <= length_edges(d))/length(dist_all_edges);
                end
                norm_sum_lengths = sum(F_l);
                p = irwin_hall_cdf(length(length_edges) + length(chain), norm_sum_lengths + norm_sum_alphas);
                
                %%
                nfa_chain = Ntests*p; % compute NFA(chain)
                
                all_nfas = [all_nfas, nfa_chain];
                if nfa_chain < best_nfa,
                    best_nfa = nfa_chain;
                    best_chain = chain;
                end
            end
        end 
    end
end

best_curve_pos = points(best_chain, :);
best_curve_ors = orientations(best_chain, :);