clear;
close all;
%% Directory containing the stimuli files
stimuli_directory = fullfile('..', 'stimuli_files');

%% Choice of the stimulus (choose the sequence and 4 parameters)
N = 100; % Total number of elements per image; N = 100, 200 or 600
seq_label = 'B';% Label of the sequene: 'A', 'B' or 'C'
n = 6;% number of aligned elements, between 4 and 12
j = 2;% jitter level, j = 1, 2, 4, 5, 8, 10 corresponding respectively to 
      % alpha = 9°, 18°, 36°, 45°, 72°, 90°
s = 1; % stimulus number, integer between 1 and 4, 
%correct_stim_number;

%% Read the corresponding file and display the image

seq = sprintf('N%d%s', N, seq_label); % name of the sequence of stimuli
stim_name = [seq, 'n', int2str(n), 'j', int2str(j), 's', int2str(s)];
points = dlmread(fullfile(stimuli_directory, seq, [stim_name, 'all_coords.txt']));
orientations = dlmread(fullfile(stimuli_directory, seq, [stim_name, 'all_ors.txt']));

% Parameters for the image
img_side = 500; % size of the image's side, in pixels
freq = 0.12; % spatial frequence for the gabor patches, in cycles per pixel
patch_side = 15; % size of each patch's side, in pixels
sigma = 1/(4*freq*patch_side);
% Build image
IMG_stimulus = gab_array(img_side, img_side, patch_side, patch_side, points, orientations, sigma, freq*patch_side);
imshow(IMG_stimulus);

%% Run the algorithm
[best_nfa best_line_pos best_line_ors all_nfas] = algo_curves(points, orientations);

%% Build and display output image
IMG_detection = gab_array(img_side, img_side, patch_side, patch_side, best_line_pos, best_line_ors, sigma, freq*patch_side);
figure;
imshow(IMG_detection);
fprintf('The detected chain has NFA = %g.\n', best_nfa);